mod traits;
mod structs;
mod impls;

extern crate time;

use traits::*;
use structs::*;

fn main() {
    println!("Hello, world!");
    
    let number: i64 = 1e3 as i64;
    println!("SinusSum Computation with {} elements", number);
    let mut serial = impls::seriall::Serial::new(0);
    let start = time::precise_time_ns();
    let result = serial.run(number,4);
    let end = time::precise_time_ns();
    println!("Get result: {} in {} ms",result,(end - start)  /1e6 as u64);

    let mut threaded_channel = impls::thread_channel::ThreadedChannel::new(0);
    let start = time::precise_time_ns();
    let result = threaded_channel.run(number,4);
    let end = time::precise_time_ns();
    println!("Get result: {} in {} ms",result,(end - start)  /1e6 as u64);

    let mut threaded_channel = impls::thread_channel::ThreadedChannel::new(0);
    let start = time::precise_time_ns();
    let result = threaded_channel.run(number,8);
    let end = time::precise_time_ns();
    println!("Get result: {} in {} ms",result,(end - start)  /1e6 as u64);

    let col = 1e3 as i32;
    //let col = 5;
    println!("Start matrix multiplication with {}x{} matrices",col,col);
    let a = Matrix::new(col,col);
    let b = Matrix::new(col,col);
    println!("Start matrix mult");
    let start = time::precise_time_ns();
    let c = a.mult(&b);
    let end = time::precise_time_ns();
    println!("Get result for the serial matrix multiplication in {} ms",(end - start)  /1e6 as u64);


    println!("Start matrix mult");
    let start = time::precise_time_ns();
    let c2 = a.parallel_mult(&b,8);
    let end = time::precise_time_ns();
    println!("Get result for the parallel matrix multiplication with 8 threads in {} ms",(end - start)  /1e6 as u64);

    println!("Start matrix mult with all rows in thread");
    let start = time::precise_time_ns();
    let c3 = a.parallel_mult_row_threads(&b);
    let end = time::precise_time_ns();
    println!("Get result for all rows in thread in {} ms",(end - start)  /1e6 as u64);

    println!("Start parallel matrix mult transpose");
    let start = time::precise_time_ns();
    let c4 = a.parallel_transpose_mult(&b,8);
    let end = time::precise_time_ns();
    println!("Get result for the parallel matrix mult transpose with 8 threads in {} ms",(end - start)  /1e6 as u64);

    println!("The matrixes are equal. This sentence is {}, {}, {}",c==c2,c2==c3,c3==c4);
}
